const xHidden = 'h-4 ml-2 opacity-10';
const xShown = 'h-4 ml-2 opacity-100';

var searchInput = document.getElementById('searchInput');
var searchAnchor = document.getElementById('searchAnchor');
if (searchInput && searchAnchor) {
  searchInput.addEventListener('input', (e) => {
    var searchterm = searchInput.value;
    if (searchterm === '') searchterm = 'setDataProperty';
    searchAnchor.href = 'https://github.com/NorthwoodsSoftware/GoJS/search?q=' + searchterm + '&type=Code';
  });

  searchInput.addEventListener('keydown', (e) => {
    if (e.key === 'Enter') searchAnchor.click();
  });
}

var selectedTag = 'all';
var filteredSamples = [];

document.addEventListener('DOMContentLoaded', (event) => changeHash());
window.addEventListener('hashchange', (event) => changeHash());

function tagOverlap(tag) {
  var mySamples = tagsDict[tag];
  var result = 0;
  for (var i = 0; i < filteredSamples.length; i++) {
    for (var j = 0; j < mySamples.length; j++) {
      if (filteredSamples[i][0] === mySamples[j]) result++;
    }
  }
  return result;
}

function buildTags() {
  var navList = document.getElementById('sidenavList');

  var alltags = Object.keys(tagsDict);

  var tagCounts = [];
  for (var i = 0; i < alltags.length; i++) {
    tagCounts.push([alltags[i], tagOverlap(alltags[i])]);
  }

  var currenttags = window.location.hash.substring(1).split(',');

  tagCounts.sort((t1, t2) => t2[1] - t1[1]); // most tags are sorted by their counts
  tagCounts.sort((t1, t2) => (currenttags.includes(t1[0]) ? -1 : 1)); // current tags appear before other tags
  tagCounts.unshift(['all', allSamples.length]); // 'all' appears first

  // clear navList
  while (navList.firstChild) {
    navList.removeChild(navList.firstChild);
  }

  // repopulate navList
  for (var i = 0; i < tagCounts.length; i++) {
    var tag = tagCounts[i][0];
    var count = tagCounts[i][1];

    if (count === 0) continue; // don't show tags with 0 samples

    var tagLink = document.createElement('a');
    tagLink.id = tag + 'Link';
    var tagsTo = window.location.hash.substring(1).split(',');
    if (tagsTo.length === 1 && tagsTo[0] === '') tagsTo = [];
    if (tag === 'all') {
      tagsTo = [];
    } else if (currenttags.includes(tag)) {
      tagsTo.splice(tagsTo.indexOf(tag), 1);
    } else {
      tagsTo.push(tag);
    }
    tagLink.href = '#' + tagsTo.join(',');
    navList.appendChild(tagLink);

    var tagCheck = document.createElement('input');
    tagCheck.type = 'checkbox';
    tagCheck.className = 'mr-2 cursor-pointer accent-nwoods-accent';
    if (currenttags.includes(tag) && tag !== 'all') {
      tagCheck.checked = true;
    }
    tagLink.appendChild(tagCheck);

    // change names of some tags to fit in side bar and be more readable
    var tagString = tag;
    if (tagString === 'layereddigraphlayout') tagString = 'layered-digraph';
    if (tagString === 'forcedirectedlayout') tagString = 'force-directed';

    var tagText = document.createElement('span');
    if (currenttags.includes(tag) && tag !== 'all') {
      tagText.innerHTML = tagString;
    } else {
      tagText.innerHTML = tagString + ' (' + count + ')';
    }
    tagLink.appendChild(tagText);
  }
}

function changeHash() {
  filteredSamples = filterSamples();
  buildTags();
  buildSamples();
}

function buildSamples() {
  var sampleListDiv = document.getElementById('samplesDiv');
  sampleListDiv.innerHTML = ''; // empty the div to start off ??

  if (filteredSamples.length === 0) return;

  for (var i = 0; i < filteredSamples.length; i++) {
    var sample = filteredSamples[i];
    var samplediv = document.createElement('div');
    samplediv.className = 'group relative before:absolute before:-inset-2.5 before:rounded-lg before:bg-gray-50 before:opacity-0 hover:before:opacity-100';
    var imgdiv = document.createElement('div');
    imgdiv.className = 'relative aspect-square overflow-hidden rounded-lg bg-gray-100 ring-1 ring-gray-900/10';

    var img = document.createElement('img');
    img.src = '../assets/images/screenshots/' + sample[3] + '.png';
    img.alt = sample[2];
    img.className = 'absolute inset-0 h-full w-full';
    imgdiv.append(img);

    var content = document.createElement('div');
    content.className = 'mt-4';

    var header = document.createElement('h4');
    header.className = 'mt-4';

    var a = document.createElement('a');
    a.className = '!border-0 hover:no-underline';
    a.href = sample[0] + '.html';
    header.append(a);

    var span = document.createElement('span');
    span.className = 'absolute -inset-2.5 z-10';
    a.append(span);

    span = document.createElement('span');
    span.className = 'relative text-lg font-semibold leading-7 text-gray-900 group-hover:text-nwoods-accent';
    span.textContent = sample[1];
    a.append(span);

    var desc = document.createElement('p');
    desc.className = 'relative mt-1.5 text-base leading-7 text-gray-600';
    desc.textContent = sample[2];

    content.append(header);
    content.append(desc);

    samplediv.append(imgdiv);
    samplediv.append(content);
    sampleListDiv.append(samplediv);
  }
}

function filterSamples() {
  var _filteredSamples = [];
  var hash = window.location.hash.substring(1).split(',');
  for (var i = 0; i < allSamples.length; i++) {
    var sample = allSamples[i];

    for (var j = 0; j < hash.length; j++) {
      var tag = hash[j];
      var toInclude = true;
      if (!tagsDict.hasOwnProperty(tag)) continue;
      if (!tagsDict[tag].includes(sample[0])) {
        toInclude = false;
        break;
      }
    }
    if (toInclude) _filteredSamples.push(sample);
  }
  return _filteredSamples;
}
